#========================================================================================
# package      DE_EPAGES::Core::t::unit::Error
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Core::API::Error
#========================================================================================
package DE_EPAGES::Core::t::unit::Error;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error qw ( QuoteStringForError );
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_QuoteStringForError_NormalString();
    $self->test_QuoteStringForError_StringWithNonPrintableChars();

    done_testing();
}



sub test_QuoteStringForError_NormalString {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AsciiString = '54657374';

    my $String = pack 'H*', $AsciiString;
    my @QuotedStringData = QuoteStringForError($String);
    my $CleanedString = $QuotedStringData[0];
    my $aFoundNonPrintableChars = $QuotedStringData[1];

    is($CleanedString, 'Test', "$SubTestName - test if text is returned unchanged");
    is_deeply($aFoundNonPrintableChars, [], "$SubTestName - test if no non-printable-character were found");
}



sub test_QuoteStringForError_StringWithNonPrintableChars {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AsciiString = '54657374001B546573741654657374';

    my $String = pack 'H*', $AsciiString;
    my @QuotedStringData = QuoteStringForError($String);
    my $CleanedString = $QuotedStringData[0];
    my $aFoundNonPrintableChars = $QuotedStringData[1];

    is($CleanedString, 'Test\x{00}\x{1B}Test\x{16}Test', "$SubTestName - test if non-printable chars were escaped correctly");
    is_deeply($aFoundNonPrintableChars, ['00', '16', '1B'], "$SubTestName - test if non-printable-chars were found");
}

__PACKAGE__->runScript();