#========================================================================================
# package      DE_EPAGES::Event::t::unit::Logger
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Event::API::Logger
#========================================================================================
package DE_EPAGES::Event::t::unit::Logger;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_logCommunication_logEnabled();
    $self->test_logCommunication_logDisabled();

    done_testing();
}



sub test_logCommunication_logEnabled {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsLogEnabled = 1;
    my $Mocks = $self->_createMocks($IsLogEnabled);
    my $Logger = $Mocks->get('SUT');

    $Logger->logCommunication('testRequest', {'test' => 'data'});

    my $WasLogged = WasCalled($Logger, '_LogCommunication');
    ok($WasLogged, "$SubTestName - test if data was logged");

    my $aLogCommunicationArgs = GetParametersFromMockifyCall($Logger, '_LogCommunication');
    is($aLogCommunicationArgs->[0], 'testRequest', "$SubTestName - test if right name was logged");
    is_deeply($aLogCommunicationArgs->[1], {'test' => 'data'}, "$SubTestName - test if right structure was logged");
    is($aLogCommunicationArgs->[2], 'Event', "$SubTestName - test if right category was logged");
}



sub test_logCommunication_logDisabled {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsLogEnabled = 0;
    my $Mocks = $self->_createMocks($IsLogEnabled);
    my $Logger = $Mocks->get('SUT');

    $Logger->logCommunication('testRequest', {'test' => 'data'});

    my $WasLogged = WasCalled($Logger, '_LogCommunication');
    ok(!$WasLogged, "$SubTestName - test if data was not logged");
}



sub _createMocks {
    my $self = shift;
    my ($IsLogEnabled) = @_;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['System'], sub {
        my ($System) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Event::API::Logger', []);
        $Mockify->addMockWithReturnValue('_LoadRootObject', $System);
        $Mockify->addMockWithReturnValueAndParameterCheck('_LogCommunication', undef, ['string', 'hashref', 'string']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('System', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Site', {
            'EventCommunicationLog' => $IsLogEnabled,
        });
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}

__PACKAGE__->runScript();