#==================================================================================================
# package      DE_EPAGES::GuidedTour::t::unit::GuidedTourHandler
# state        public
#--------------------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::GuidedTour::t::unit::GuidedTourHandler
#==================================================================================================
package DE_EPAGES::GuidedTour::t::unit::GuidedTourHandler;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::GuidedTour::API::TLE::GuidedTourHandler;
use DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler;
use DE_EPAGES::GuidedTour::t::unit::MockupObjects qw (NewShop NewViewAction);
use Test::More;

use strict;

sub test {
    my $self = shift;

    # test getActiveTours
    $self->testGetActiveTours_InvalidShopType();
    $self->testGetActiveTours_InvalidPageType();
    $self->testGetActiveTours_UndefTours();
    $self->testGetActiveTours_NoActiveTour();

    # test createTourForShop
    $self->testCreateTourForShop_UndefinedShop();
    $self->testCreateTourForShop_UndefinedTour();
    $self->testCreateTourForShop_NewTour();
    $self->testCreateTourForShop_ExistingTour();

    # test getViewActionAlias
    $self->testgetViewActionAlias_UndefinedPageTypeAlias();
    $self->testgetViewActionAlias_InvalidPageTypeAlias();
    $self->testgetViewActionAlias_ValidPageTypeAlias();

    # test getShopTour
    $self->testgetShopTour_InvalidTourAlias();
    $self->testgetShopTour_ValidTourAlias();
    $self->testgetShopTour_NotExistingTourForShop();

    # test getViewActionObjectID
    $self->testgetViewActionObjectID_UndefinedViewActionObjectAlias();

    #test getShopTour
    $self->testgetShopTour_ValidViewActionObjectAliasAttribute();
    $self->testgetShopTour_ValidViewActionObjectAliasChild();

    done_testing();
}

sub testgetShopTour_ValidViewActionObjectAliasChild {
    my $self = shift;

    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();
    my $ViewActionObjectAlias = 'UnityMBO-ViewShop';
    my $Shop                  = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
        {
                'Child' => { $ViewActionObjectAlias => NewViewAction()}
        }
    );

    my $ViewActionID = $GuidedTourHandler->getViewActionObjectID($Shop,$ViewActionObjectAlias);

    is($ViewActionID, '12345', 'getViewActionObjectID corresponding ViewActionObjectAlias Child');

    return;
}

sub testgetShopTour_ValidViewActionObjectAliasAttribute {
    my $self = shift;

    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();
    my $ViewActionObjectAlias = 'UnityMBO-ViewShop';
    my $Shop                  = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
        {
                $ViewActionObjectAlias => NewViewAction(),
                'Child' => DE_EPAGES::Test::Mock::EpagesObject->new(
                    'DE_EPAGES::Shop::API::Object::Shop',
                        {
                             'Alias' => 'TestObject',
                        }
                    )
        }
    );

    my $ViewActionID = $GuidedTourHandler->getViewActionObjectID($Shop,$ViewActionObjectAlias);

    is($ViewActionID, '12345', 'getViewActionObjectID corresponding ViewActionObjectAlias Attribute');

    return;
}

sub testgetViewActionObjectID_UndefinedViewActionObjectAlias {
    my $self = shift;

    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();
    my $ViewActionObjectAlias = '';
    my $Shop                  = NewShop();

    eval {
        $GuidedTourHandler->getViewActionObjectID($Shop,$ViewActionObjectAlias);
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'INVALID_PARAM', 'getViewActionObjectID undefined ViewActionObjectAlias');
    }
    else {
        fail('getViewActionObjectID undefined ViewActionObjectAlias');
    }

    return;
}

sub testgetShopTour_InvalidTourAlias {
    my $self = shift;
    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();

    my $Shop        = NewShop();

    eval {
        $GuidedTourHandler->getShopTour($Shop,'');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'INVALID_PARAM', 'getShopTour invalid TourAlias');
    }
    else {
        fail('getShopTour invalid TourAlias');
    }
    return;
}

sub testgetShopTour_ValidTourAlias {
    my $self = shift;
    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();

    my $hTourInShop = { 'Alias' => 'GuidedTourGeneral' };
    my $Shop        = NewShop({
        'UnityMBOGuidedTours' => [ $hTourInShop ]
    });

    my $ShopTour = $GuidedTourHandler->getShopTour($Shop,'GuidedTourGeneral');

    is($ShopTour->get('Alias'),'GuidedTourGeneral', 'getShopTour corresponding TourAlias');

    return;
}

sub testgetShopTour_NotExistingTourForShop {
    my $self = shift;
    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();

    my $Shop        = NewShop();

    my $ShopTour = $GuidedTourHandler->getShopTour($Shop,'GuidedTourGeneral');

    is($ShopTour,undef,'getShopTour not existing TourAlias');

    return;
}

sub testgetViewActionAlias_UndefinedPageTypeAlias {
    my $self = shift;
    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();

    eval {
        $GuidedTourHandler->getViewActionAlias();
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'INVALID_PARAM', 'getViewActionAlias undef action alias');
    }
    else {
        fail('getViewActionAlias undef action alias');
    }

    return;
}

sub testgetViewActionAlias_InvalidPageTypeAlias {
    my $self = shift;
    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();

    eval {
        $GuidedTourHandler->getViewActionAlias('');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'INVALID_PARAM', 'getViewActionAlias invalid action alias');
    }
    else {
        fail('getViewActionAlias invalid action alias');
    }

    return;
}

sub testgetViewActionAlias_ValidPageTypeAlias {
    my $self = shift;
    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();

    my $ViewActionAlias = $GuidedTourHandler->getViewActionAlias('UnityMBO-Shop');

    is($ViewActionAlias,'UnityMBO-ViewMBO', 'getViewActionAlias valid action alias');

    return;
}

sub testCreateTourForShop_UndefinedShop {
    my $self = shift;

    my $hTour             = { 'Alias' => 'GuidedTourGeneral' };
    my $Shop              = undef;
    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();

    eval {
        $GuidedTourHandler->createTourForShop($Shop, $hTour);
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'INVALID_PARAM', 'createTourForShop undefined shop');
    }
    else {
        fail('createTourForShop undefined shop');
    }

    return;
}

sub testCreateTourForShop_UndefinedTour {
    my $self = shift;

    my $hTour             = undef;
    my $Shop              = NewShop();
    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();

    eval {
        $GuidedTourHandler->createTourForShop($Shop, $hTour);
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'INVALID_PARAM', 'createTourForShop undefined tour');
    }
    else {
        fail('createTourForShop undefined tour');
    }

    return;
}

sub testCreateTourForShop_NewTour {
    my $self = shift;

    my $hTour       = { 'Alias' => 'GuidedTourGeneral' };
    my $Shop        = NewShop({
        'UnityMBOGuidedTours' => []
    });
    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();
    my $ResultTour = $GuidedTourHandler->createTourForShop($Shop, $hTour);

    is($ResultTour->alias, $hTour->{'Alias'}, 'createTourForShop new tour');

    return;
}

sub testCreateTourForShop_ExistingTour {
    my $self = shift;

    my $hTour       = { 'Alias' => 'GuidedTourGeneral' };
    my $hTourInShop = { 'Alias' => 'GuidedTourGeneral' };
    my $Shop        = NewShop({
        'UnityMBOGuidedTours' => [ $hTourInShop ]
    });
    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();
    my $ResultTour = $GuidedTourHandler->createTourForShop($Shop, $hTour);

    is($ResultTour->alias, $hTourInShop->{'Alias'}, 'createTourForShop existing tour');

    return;
}
sub testGetActiveTours_InvalidShopType {
    my $self = shift;

    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();
    my $ShopTypeAlias     = '';
    my $PageTypeAlias     = 'UnityMBO-Shop';
    my $aGuidedTours      = [];
    my $hParams         = { 'ShopTypeAlias' => $ShopTypeAlias,
                            'PageTypeAlias' => $PageTypeAlias,
    };

    my $ActiveTours = $GuidedTourHandler->getActiveTours($aGuidedTours,$hParams);

    is($ActiveTours->[0], undef, 'getActiveTours invalid shop type');

    return;
}

sub testGetActiveTours_InvalidPageType {
    my $self = shift;

    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();
    my $ShopTypeAlias     = 'eCom5000';
    my $PageTypeAlias     = '';
    my $aGuidedTours      = [];
    my $hParams         = { 'ShopTypeAlias' => $ShopTypeAlias,
                            'PageTypeAlias' => $PageTypeAlias,
    };

    my $ActiveTours = $GuidedTourHandler->getActiveTours($aGuidedTours,$hParams);

    is($ActiveTours->[0], undef, 'getActiveTours invalid shop type');

    return;
}

sub testGetActiveTours_UndefTours {
    my $self = shift;

    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();
    my $ShopTypeAlias     = 'eCom5000';
    my $PageTypeAlias     = 'UnityMBO-Shop';
    my $aGuidedTours      = undef;
    my $hParams         = { 'ShopTypeAlias' => $ShopTypeAlias,
                            'PageTypeAlias' => $PageTypeAlias,
    };

    my $ahTours = $GuidedTourHandler->getActiveTours($aGuidedTours,$hParams);

    is(scalar @$ahTours, 0, 'getActiveTours no active tour');

    return;
}

sub testGetActiveTours_NoActiveTour {
    my $self = shift;

    my $GuidedTourHandler = DE_EPAGES::GuidedTour::t::unit::MockedGuidedTourHandler->new();
    my $ShopTypeAlias     = 'eCom5000';
    my $PageTypeAlias     = 'UnityMBO-Shop';
    my $aGuidedTours      = [];
    my $hParams         = { 'ShopTypeAlias' => $ShopTypeAlias,
                            'PageTypeAlias' => $PageTypeAlias,
    };

    my $ahTours = $GuidedTourHandler->getActiveTours($aGuidedTours,$hParams);

    is(scalar @$ahTours, 0, 'getActiveTours no active tour');

    return;
}

__PACKAGE__->runScript();
