#==================================================================================================
# package      DE_EPAGES::GuidedTour::t::unit::GuidedTourWordingHandler
# state        public
#--------------------------------------------------------------------------------------------------
# description  Tests for DE_EPAGES::GuidedTour::t::unit::GuidedTourWordingHandler
#==================================================================================================
package DE_EPAGES::GuidedTour::t::unit::GuidedTourWordingHandler;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error                                                     qw (ExistsError
                                                                                        GetError);
use DE_EPAGES::GuidedTour::API::TLE::GuidedTourWordingHandler;
use DE_EPAGES::GuidedTour::t::unit::MockupObjects                          qw (NewGuidedTourWordingHandler
                                                                                        NewGuidedTourConfig
                                                                                        NewUser);

use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->testGetTranslation_UndefinedTour();
    $self->testGetTranslation_UndefinedUser();
    $self->testGetTranslation_InvalidTemplate();
    $self->testGetTranslation_MissingWording();
    $self->testGetTranslation_Success();

    done_testing();
}

sub testGetTranslation_UndefinedTour {
    my $self = shift;

    my $GuidedTour     = undef;
    my $User           = NewUser();
    my $WordingHandler = NewGuidedTourWordingHandler({'WelcomeMessage' => 'TestTranslation'});

    eval { $WordingHandler->getTranslation('WelcomeMessage', $GuidedTour, $User) };
    if ( ExistsError() ) {
        my $Error = GetError();
        is( $Error->code, 'INVALID_PARAM', 'getTranslation undefined tour' );
    }
    else {
        fail('getTranslation undefined tour');
    }

    return;
}

sub testGetTranslation_UndefinedUser {
    my $self = shift;

    my $GuidedTour = NewGuidedTourConfig(
        {
            'Alias'        => 'TestTour1',
            'TemplateName' => 'TestTemplate',
        }
    );
    my $User           = undef;
    my $WordingHandler = NewGuidedTourWordingHandler({'WelcomeMessage' => 'TestTranslation'});

    eval { $WordingHandler->getTranslation('WelcomeMessage', $GuidedTour, $User) };
    if ( ExistsError() ) {
        my $Error = GetError();
        is( $Error->code, 'INVALID_PARAM', 'getTranslation undefined user' );
    }
    else {
        fail('getTranslation undefined user');
    }

    return;
}

sub testGetTranslation_InvalidTemplate {
    my $self = shift;

    my $GuidedTour = NewGuidedTourConfig(
        {
            'Alias'        => 'TestTour1',
            'TemplateName' => '',
        }
    );
    my $User           = NewUser();
    my $WordingHandler = NewGuidedTourWordingHandler({'WelcomeMessage' => 'TestTranslation'});

    eval { $WordingHandler->getTranslation('WelcomeMessage', $GuidedTour, $User) };
    if ( ExistsError() ) {
        my $Error = GetError();
        ok( ($Error->code eq 'INVALID_TEAMPLATE_NAME' or $Error->code eq 'AttributeNotExists'),
            'getTranslation invalid template' );
    }
    else {
        fail('getTranslation invalid template');
    }

    return;
}

sub testGetTranslation_MissingWording {
    my $self = shift;

    my $GuidedTour = NewGuidedTourConfig(
        {
            'Alias'        => 'TestTour1',
            'TemplateName' => 'TestTemplate',
            'Class'        => DE_EPAGES::Test::Mock::EpagesObject->new(
                'DE_EPAGES::Object::API::Object::Object',
                    {
                    'Package' => 'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',
                    }
                ),
        }
    );
    my $User           = NewUser();
    my $WordingHandler = NewGuidedTourWordingHandler({'WelcomeMessage' => ''});

    eval { $WordingHandler->getTranslation('WelcomeMessage', $GuidedTour, $User) };
    if ( ExistsError() ) {
        my $Error = GetError();
        is( $Error->code, 'MISSING_TRANSLATION', 'getTranslation missing translation' );
    }
    else {
        fail('getTranslation missing translation');
    }

    return;
}

sub testGetTranslation_Success {
    my $self = shift;

    my $GuidedTour = NewGuidedTourConfig(
        {
            'Alias'        => 'TestTour1',
            'TemplateName' => 'TestTemplate',
            'Class'        => DE_EPAGES::Test::Mock::EpagesObject->new(
                'DE_EPAGES::Object::API::Object::Object',
                    {
                    'Package' => 'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',
                    }
                ),
        }
    );
    my $User = NewUser();
    my $WordingHandler = NewGuidedTourWordingHandler({'WelcomeMessage' => 'TestTranslation'});

    my $Result = $WordingHandler->getTranslation('WelcomeMessage', $GuidedTour, $User);

    is( $Result, 'TestTranslation', 'getTranslation success' );

    return;
}

__PACKAGE__->runScript();
