#==================================================================================================
# package      DE_EPAGES::GuidedTour::t::unit::UnityMBOGuidedTourConfig
# state        public
#--------------------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::GuidedTour::t::unit::UnityMBOGuidedTourConfig
#==================================================================================================
package DE_EPAGES::GuidedTour::t::unit::UnityMBOGuidedTourConfig;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig;
use DE_EPAGES::Core::API::Error                                         qw (ExistsError GetError);
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_classname();

    # Test method isCorrectShopType
    $self->testisCorrectShopType_UndefinedShopTypeAlias();
    $self->testisCorrectShopType_InvalidShopTypeAlias();
    $self->testisCorrectShopType_ValidShopTypeAlias();
    $self->testisCorrectShopType_ExcludedShopTypeAlias();
    $self->testisCorrectShopType_ValidShopTypeAliasNotAll();

    # test method isCorrectPageType
    $self->testisCorrectPageType_UndefinedPageTypeAlias();
    $self->testisCorrectPageType_InvalidPageTypeAlias();
    $self->testisCorrectPageType_InValidPageTypeAlias2();
    $self->testisCorrectPageType_InValidPageTypeAlias3();
    $self->testisCorrectPageType_ValidPageTypeAlias();

    # test additional criteria
    $self->testadditionalCriteria();

    #test ensurePreconditions
    $self->testensurePreConditions();

    done_testing();
}

sub test_classname {
    my $self = shift;

    my $GuidedTour          = DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig->new();
    my $GuidedTourClassName = $GuidedTour->className;
    is($GuidedTourClassName, 'UnityMBOGuidedTourConfig', 'guided tour classname ok');

    return;
}

sub testisCorrectShopType_UndefinedShopTypeAlias {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',[]);
    my $TestTourConfig = $Mockify->getMockObject();

    eval {
        $TestTourConfig->isCorrectShopType(undef);
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'INVALID_PARAM', 'iscorrectshoptype undefined ShopTypeAlias');
    }
    else {
        fail('iscorrectshoptype undefined ShopTypeAlias');
    }

    return;
}

sub testisCorrectShopType_InvalidShopTypeAlias {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',[]
    );
    my $TestTourConfig = $Mockify->getMockObject();

    eval {
        $TestTourConfig->isCorrectShopType('');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'INVALID_PARAM', 'iscorrectshoptype undefined PageTypeAlias');
    }
    else {
        fail('iscorrectshoptype undefined PageTypeAlias');
    }

    return;
}

sub testisCorrectShopType_ValidShopTypeAlias {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',[]
    );

    $Mockify->addMockWithReturnValue('getAssignedShopType', 'all');
    $Mockify->addMockWithReturnValue('getExcludedShopType', '');

    my $TestTourConfig = $Mockify->getMockObject();

    my $IsCorrectShopType = $TestTourConfig->isCorrectShopType('Test');

    is($IsCorrectShopType,'1','iscorrectshoptype valid shoptypealias');

    return;
}
sub testisCorrectShopType_ValidShopTypeAliasNotAll {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',[]
    );

    $Mockify->addMockWithReturnValue('getAssignedShopType', 'Test');
    $Mockify->addMockWithReturnValue('getExcludedShopType', '');

    my $TestTourConfig = $Mockify->getMockObject();

    my $IsCorrectShopType = $TestTourConfig->isCorrectShopType('Test');

    is($IsCorrectShopType,'1','iscorrectshoptype valid shoptypealias (notAll)');

    return;
}

sub testisCorrectShopType_ExcludedShopTypeAlias {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',[]
    );

    $Mockify->addMockWithReturnValue('getAssignedShopType', 'all');
    $Mockify->addMockWithReturnValue('getExcludedShopType', 'Test');

    my $TestTourConfig = $Mockify->getMockObject();

    my $IsCorrectShopType = $TestTourConfig->isCorrectShopType('Test');

    is($IsCorrectShopType,'0','iscorrectshoptype excluded shoptypealias');

    return;
}

sub testisCorrectPageType_UndefinedPageTypeAlias {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',[]
    );
    my $TestTourConfig = $Mockify->getMockObject();

    eval {
        $TestTourConfig->isCorrectPageType(undef);
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'INVALID_PARAM', 'iscorrectpagetype undefined PageTypeAlias');
    }
    else {
        fail('iscorrectpagetype undefined PageTypeAlias');
    }

    return;
}

sub testisCorrectPageType_InvalidPageTypeAlias {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',[]
    );
    my $TestTourConfig = $Mockify->getMockObject();

    eval {
        $TestTourConfig->isCorrectPageType('');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'INVALID_PARAM', 'iscorrectpagetype invalid PageTypeAlias');
    }
    else {
        fail('iscorrectpagetype invalid PageTypeAlias');
    }

    return;
}

sub testisCorrectPageType_ValidPageTypeAlias {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',[]
    );

    $Mockify->addMockWithReturnValue('getAssignedPageTypes', 'MBO-Test,MBO-GeneralTest');

    my $TestTourConfig = $Mockify->getMockObject();

    my $IsCorrectPageType = $TestTourConfig->isCorrectPageType('MBO-Test');

    is($IsCorrectPageType,'1','iscorrectpagetype valid pagetype alias');

    return;
}

sub testisCorrectPageType_InValidPageTypeAlias2 {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',[]
    );

    $Mockify->addMockWithReturnValue('getAssignedPageTypes', 'MBO-Test,MBO-GeneralTest');

    my $TestTourConfig = $Mockify->getMockObject();

    my $IsCorrectPageType = $TestTourConfig->isCorrectPageType('MBO-Te');

    is($IsCorrectPageType,'0','iscorrectpagetype valid pagetype alias');

    return;
}

sub testisCorrectPageType_InValidPageTypeAlias3 {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig',[]
    );

    $Mockify->addMockWithReturnValue('getAssignedPageTypes', '');

    my $TestTourConfig = $Mockify->getMockObject();

    my $IsCorrectPageType = $TestTourConfig->isCorrectPageType('MBO-Test');

    is($IsCorrectPageType,'0','iscorrectpagetype valid pagetype alias');

    return;
}

sub testadditionalCriteria {
    my $self = shift;

    my $GuidedTour          = DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig->new();
    my $GuidedTourAdditionalCriteria = $GuidedTour->additionalCriteria;
    is($GuidedTourAdditionalCriteria, '1', 'additionalcriteria ok');

    return;
}

sub testensurePreConditions {
    my $self = shift;

    my $GuidedTour          = DE_EPAGES::GuidedTour::API::Object::UnityMBOGuidedTourConfig->new();
     $GuidedTour->ensurePreConditions;
    ok(1,'ensurepreconditions ok');

    return;
}

__PACKAGE__->runScript();
