#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::PatchPayment_AddressRepository
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for PatchPayment::AddressRepository
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::PatchPayment_AddressRepository;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::AddressDTO;
use DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::AddressRepository;
use DE_EPAGES::Test::Mock::EpagesObject;

use Test::More;
use strict;

sub test {
    my $self = shift;

    $self->test_Successful_CollectData();

    done_testing();
}

sub test_Successful_CollectData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingAddress = $self->_createShippingAddress();

    my $ShippingAddressRepository = DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::AddressRepository->new();
    ok($ShippingAddressRepository->collectData($ShippingAddress), "$SubTestName - test if the collect is successful");

    my $ShippingAddressDTO = $ShippingAddressRepository->getAddressDTO();
    is($ShippingAddressDTO->recipientName() , 'John Snow', "$SubTestName - test if name was set correctly");
    is($ShippingAddressDTO->line1() , 'Black Castle Street', "$SubTestName - test if line1 was set correctly");
    is($ShippingAddressDTO->city() , 'Winterfell', "$SubTestName - test if city was set correctly");
    is($ShippingAddressDTO->countryState() , 'SN', "$SubTestName - test if state was set correctly");
    is($ShippingAddressDTO->postalCode() , 'Beyond the wall', "$SubTestName - test if postal code was set correctly");
    is($ShippingAddressDTO->countryCode() , 'You know nothing!', "$SubTestName - test if country code was set correctly");

    my $hExpectedShippingAddress = {
          'city' => 'Winterfell',
          'state' => 'SN',
          'recipientName' => 'John Snow',
          'line1' => 'Black Castle Street',
          'countryCode' => 'You know nothing!',
          'postalCode' => 'Beyond the wall'
    };
    is_deeply($ShippingAddressDTO->toHash() , $hExpectedShippingAddress, "$SubTestName - test if hash was set correctly");

}

#---------------------------------------------------------------------------
#  Helpers
#---------------------------------------------------------------------------
sub _createShippingAddress {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'ShippingAddress',
        'Country' => {'Code2' => 'You know nothing!'},
        'City' => 'Winterfell',
        'State' => 'SN',
        'FullName' => 'John Snow',
        'Street' => 'Black Castle Street',
        'Zipcode' => 'Beyond the wall',
    };
    my $ShippingAddressProduct = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAttributes );
    return $ShippingAddressProduct;
}

__PACKAGE__->runScript();