#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::UI_JSON
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::JSON
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::UI_JSON;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use Test::Exception;
use DE_EPAGES::Test::Mock::Mockify qw (GetParametersFromMockifyCall WasCalled);
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Order::API::Object::Basket;
use DE_EPAGES::Test::Mock::EpagesObject;
use strict;

sub test {
    my $self = shift;
    $self->{'ToCheck'} = {};
    $self->test_CreatePaymentPayPalPlus();
    $self->test_CreatePaymentPayPalPlus_NoAddress();
    $self->test_CreatePaymentPayPalPlus_CreatePaymentError();
    $self->test_CreatePaymentPayPalPlus_PatchPaymentError();
    $self->test_GetPaymentWallDataPayPalPlus();
    $self->test_GetPaymentWallDataPayPalPlus_Error();
    $self->test_PatchPaymentPayPalPlus();
    $self->test_PatchPaymentPayPalPlus_Error();

    done_testing();
}

sub test_CreatePaymentPayPalPlus {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->_createServlet();
    my $UI_JSON = $self->_buildUIJSON();

    lives_ok( sub { $UI_JSON->CreatePaymentPayPalPlus($Servlet) }, "$SubTestName - Test if CreatePaymentPayPalPlus lives" );

    my $aParametersFromCall = GetParametersFromMockifyCall($UI_JSON, 'writeContentOutput', 0);
    my $hJSONResult = $aParametersFromCall->[0];

    my $hExpectedJSONResult = {
        'WasCreatePaymentSuccessful' => 1,
    };
    is_deeply($hJSONResult, $hExpectedJSONResult, "$SubTestName - Test if WasCreatePaymentSuccessful is 1 and CreatePaymentErrorCode and CreatePaymentErrorMessage are undef");

    my $aParametersFromReturnUrlCall = GetParametersFromMockifyCall($UI_JSON, '_BuildShopUrl', 0);
    my $hReturnUrlParams = $aParametersFromReturnUrlCall->[1];
    my $hExpectedReturnUrlParams = {
        'ChangeAction' => 'PaymentBacklinkPayPalPlus',
        'PaymentLineItemGUID' => 'Some-GUID',
    };
    is_deeply($hReturnUrlParams, $hExpectedReturnUrlParams, "$SubTestName - Test if _buildReturnUrl was called with the correct parameters");

    my $aParametersFromCancelUrlCall = GetParametersFromMockifyCall($UI_JSON, '_BuildShopUrl', 1);
    my $hCancelUrlParams = $aParametersFromCancelUrlCall->[1];
    my $hExpectedCancelUrlParams = {
        'ViewAction' => 'ViewPayPalPlusCancelPayment',
        'PaymentLineItemGUID' => 'Some-GUID',
    };
    is_deeply($hCancelUrlParams, $hExpectedCancelUrlParams, "$SubTestName - Test if _buildCancelUrl was called with the correct parameters");

    my $ContextLogger = $UI_JSON->{'__ToCheck'}->{'_ContextLogger'};
    my $aParams_logContext = $ContextLogger->{'__ToCheck'}->{'_Params_logContext'};
    my $BasketObject = @{$aParams_logContext}[0];
    my $MethodName = @{$aParams_logContext}[1];
    my $FunctionPointer = @{$aParams_logContext}[2];

    is($BasketObject->id(), 'BasketID', "$SubTestName - Test if the basket object was given to _logContext");
    ok($BasketObject->isa('DE_EPAGES::Order::API::Object::Basket'), "$SubTestName - Test if the basket object was given to _logContext");
    is($MethodName, 'DE_EPAGES::PayPalPlus::UI::JSON::CreatePaymentPayPalPlus', "$SubTestName - Test if the string was given to _logContext");
    is(ref $FunctionPointer, 'CODE', "$SubTestName - Test if the function was given to _logContext");

    my $WasCreatePaymentCalled = WasCalled($self->{'ToCheck'}->{'CreatePaymentService'}, 'doRequest');
    is($WasCreatePaymentCalled, 1, "$SubTestName - Test if Create Payment was called");
    my $WasPatchPaymentCalled = WasCalled($self->{'ToCheck'}->{'PatchPaymentService'}, 'doRequest');
    is($WasPatchPaymentCalled, 1, "$SubTestName - Test if Patch Payment was called");
}

sub test_CreatePaymentPayPalPlus_NoAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->_createServlet_WithNoAddress();
    my $UI_JSON = $self->_buildUIJSON();

    lives_ok( sub { $UI_JSON->CreatePaymentPayPalPlus($Servlet) }, "$SubTestName - Test if CreatePaymentPayPalPlus lives" );

    my $aParametersFromCall = GetParametersFromMockifyCall($UI_JSON, 'writeContentOutput', 0);
    my $hJSONResult = $aParametersFromCall->[0];

    my $hExpectedJSONResult = {
        'WasCreatePaymentSuccessful' => 1,
    };
    is_deeply($hJSONResult, $hExpectedJSONResult, "$SubTestName - Test if WasCreatePaymentSuccessful is 1 and CreatePaymentErrorCode and CreatePaymentErrorMessage are undef");

    my $aParametersFromReturnUrlCall = GetParametersFromMockifyCall($UI_JSON, '_BuildShopUrl', 0);
    my $hReturnUrlParams = $aParametersFromReturnUrlCall->[1];
    my $hExpectedReturnUrlParams = {
        'ChangeAction' => 'PaymentBacklinkPayPalPlus',
        'PaymentLineItemGUID' => 'Some-GUID',
    };
    is_deeply($hReturnUrlParams, $hExpectedReturnUrlParams, "$SubTestName - Test if _buildReturnUrl was called with the correct parameters");

    my $aParametersFromCancelUrlCall = GetParametersFromMockifyCall($UI_JSON, '_BuildShopUrl', 1);
    my $hCancelUrlParams = $aParametersFromCancelUrlCall->[1];
    my $hExpectedCancelUrlParams = {
        'ViewAction' => 'ViewPayPalPlusCancelPayment',
        'PaymentLineItemGUID' => 'Some-GUID',
    };
    is_deeply($hCancelUrlParams, $hExpectedCancelUrlParams, "$SubTestName - Test if _buildCancelUrl was called with the correct parameters");

    my $ContextLogger = $UI_JSON->{'__ToCheck'}->{'_ContextLogger'};
    my $aParams_logContext = $ContextLogger->{'__ToCheck'}->{'_Params_logContext'};
    my $BasketObject = @{$aParams_logContext}[0];
    my $MethodName = @{$aParams_logContext}[1];
    my $FunctionPointer = @{$aParams_logContext}[2];

    is($BasketObject->id(), 'BasketID', "$SubTestName - Test if the basket object was given to _logContext");
    ok($BasketObject->isa('DE_EPAGES::Order::API::Object::Basket'), "$SubTestName - Test if the basket object was given to _logContext");
    is($MethodName, 'DE_EPAGES::PayPalPlus::UI::JSON::CreatePaymentPayPalPlus', "$SubTestName - Test if the string was given to _logContext");
    is(ref $FunctionPointer, 'CODE', "$SubTestName - Test if the function was given to _logContext");

    my $WasCreatePaymentCalled = WasCalled($self->{'ToCheck'}->{'CreatePaymentService'}, 'doRequest');
    is($WasCreatePaymentCalled, 1, "$SubTestName - Test if Create Payment was called");
    my $WasPatchPaymentCalled = WasCalled($self->{'ToCheck'}->{'PatchPaymentService'}, 'doRequest');
    is($WasPatchPaymentCalled, 0, "$SubTestName - Test if Patch Payment was called");
}

sub test_CreatePaymentPayPalPlus_CreatePaymentError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->_createServlet();
    my $UI_JSON = $self->_buildUIJSON_ErrorCreatePayment();

    lives_ok( sub { $UI_JSON->CreatePaymentPayPalPlus($Servlet) }, "$SubTestName - Test if CreatePaymentPayPalPlus lives" );

    my $aParametersFromCall = GetParametersFromMockifyCall($UI_JSON, 'writeContentOutput', 0);
    my $hJSONResult = $aParametersFromCall->[0];

    is($hJSONResult->{'WasCreatePaymentSuccessful'}, 0, "$SubTestName - Test if WasCreatePaymentSuccessful is 0");
}

sub test_CreatePaymentPayPalPlus_PatchPaymentError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->_createServlet();
    my $UI_JSON = $self->_buildUIJSON_ErrorPatchPayment();

    lives_ok( sub { $UI_JSON->CreatePaymentPayPalPlus($Servlet) }, "$SubTestName - Test if CreatePaymentPayPalPlus lives" );

    my $aParametersFromCall = GetParametersFromMockifyCall($UI_JSON, 'writeContentOutput', 0);
    my $hJSONResult = $aParametersFromCall->[0];

    is($hJSONResult->{'WasCreatePaymentSuccessful'}, 0, "$SubTestName - Test if WasCreatePaymentSuccessful is 0");
}

sub test_GetPaymentWallDataPayPalPlus {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentWallDataBuilder = $self->_createPaymentWallDataBuilder();
    my $Servlet = $self->_createServlet_With_LineItemPaymentPayPalPlus();
    my $UI_JSON = $self->_buildUIJSON_With_PaymentWallDataBuilder($PaymentWallDataBuilder);

    $UI_JSON->GetPaymentWallDataPayPalPlus($Servlet);

    is(WasCalled($PaymentWallDataBuilder, 'getData'), 1, "$SubTestName - Test if getData was called" );
}

sub test_GetPaymentWallDataPayPalPlus_Error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentWallDataBuilder = $self->_createPaymentWallDataBuilder_Error();
    my $Servlet = $self->_createServlet_With_LineItemPaymentPayPalPlus();
    my $UI_JSON = $self->_buildUIJSON_With_PaymentWallDataBuilder($PaymentWallDataBuilder);

    $UI_JSON->GetPaymentWallDataPayPalPlus($Servlet);

    is(WasCalled($PaymentWallDataBuilder, 'getError'), 1, "$SubTestName - Test if getError was called" );
}

sub test_PatchPaymentPayPalPlus {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->_createServlet();
    my $UI_JSON = $self->_buildUIJSON();

    lives_ok( sub { $UI_JSON->PatchPaymentPayPalPlus($Servlet) }, "$SubTestName - Test if PatchPaymentPayPalPlus lives" );

    my $aParametersFromCall = GetParametersFromMockifyCall($UI_JSON, 'writeContentOutput', 0);
    my $hJSONResult = $aParametersFromCall->[0];

    my $hExpectedJSONResult = {
        'WasPatchPaymentSuccessful' => 1,
    };
    is_deeply($hJSONResult, $hExpectedJSONResult, "$SubTestName - Test if WasPatchPaymentSuccessful is 1");

    my $ContextLogger = $UI_JSON->{'__ToCheck'}->{'_ContextLogger'};
    my $aParams_logContext = $ContextLogger->{'__ToCheck'}->{'_Params_logContext'};
    my $BasketObject = @{$aParams_logContext}[0];
    my $MethodName = @{$aParams_logContext}[1];
    my $FunctionPointer = @{$aParams_logContext}[2];

    is($BasketObject->id(), 'BasketID', "$SubTestName - Test if the basket object was given to _logContext");
    ok($BasketObject->isa('DE_EPAGES::Order::API::Object::Basket'), "$SubTestName - Test if the basket object was given to _logContext");
    is($MethodName, 'DE_EPAGES::PayPalPlus::UI::JSON::PatchPaymentPayPalPlus', "$SubTestName - Test if the string was given to _logContext");
    is(ref $FunctionPointer, 'CODE', "$SubTestName - Test if the function was given to _logContext");

    my $WasPatchPaymentCalled = WasCalled($self->{'ToCheck'}->{'PatchPaymentService'}, 'doRequest');
    is($WasPatchPaymentCalled, 1, "$SubTestName - Test if Patch Payment was called");
}

sub test_PatchPaymentPayPalPlus_Error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->_createServlet();
    my $UI_JSON = $self->_buildUIJSON_ErrorPatchPayment();

    lives_ok( sub { $UI_JSON->PatchPaymentPayPalPlus($Servlet) }, "$SubTestName - Test if PatchPaymentPayPalPlus lives" );

    my $aParametersFromCall = GetParametersFromMockifyCall($UI_JSON, 'writeContentOutput', 0);
    my $hJSONResult = $aParametersFromCall->[0];

    is($hJSONResult->{'WasPatchPaymentSuccessful'}, 0, "$SubTestName - Test if WasPatchPaymentSuccessful is 0");
}

sub _createServlet {
    my $self = shift;
    my $Basket = $self->_buildBasketWithShippingAddress();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $Basket );
    $MockifyServlet->addVar('IsMobile', 0);
    return $MockifyServlet->getMockObject();
}

sub _createServlet_WithNoAddress {
    my $self = shift;
    my $Basket = $self->_buildBasketWithNoAddress();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $Basket );
    $MockifyServlet->addVar('IsMobile', 0);
    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'Shop',
        'ID' => 'IdOfTheShop',
        'OrderProcess' => '0'
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
}

sub _buildBasketWithShippingAddress {
    my $self = shift;
    my $hAttributes = {
        'BillingAddress' => undef,
        'ShippingAddress' => $self->_createAddress('ShippingAddress'),
    };
    my $Basket = $self->_createBasket($hAttributes);
    return $Basket;
}

sub _buildBasketWithNoAddress {
    my $self = shift;
    my $hAttributes = {
        'BillingAddress' => undef,
        'ShippingAddress' => undef,
    };
    my $Basket = $self->_createBasket($hAttributes);
    return $Basket;
}

sub _createBasket {
    my $self = shift;
    my ($hAdditionalAttributes) = @_;
    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer(),
        'ID' => 'BasketID',
        'Site' => $self->_createShop(),
        %{$hAdditionalAttributes},
    };
    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', $hAttributes );
    return $Basket;
}

sub _createAddress {
    my $self = shift;
    my ($AddressName) = @_;
    my $hAttributes = {
        'Alias' => $AddressName,
        'Name' =>  'MyTestName',
    };
    my $Address = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAttributes );
    return $Address;
}

sub _createLineItemContainer {
    my $self = shift;
    my $hAttributes = {
        'Payment' => $self->_createPayment(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);
}

sub _createPayment {
    my $self = shift;
    my $hAttributes = {
        'GUID' => 'Some-GUID',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemPayment', $hAttributes );
}

sub _createServlet_With_LineItemPaymentPayPalPlus {
    my $self = shift;
    my $hAttributes = {
        'ID'   => '12345',
        'Site' => undef
    };
    my $LineItemPaymentPayPalPlus = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus',$hAttributes);
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $LineItemPaymentPayPalPlus );
    return $MockifyServlet->getMockObject();
}

sub _buildUIJSON {
    my $self = shift;
    my $CreatePaymentService = $self->_createCreatePaymentService();
    my $PatchPaymentService = $self->_createPatchPaymentService();
    return $self->_createUIJSON($CreatePaymentService, $PatchPaymentService);
}

sub _buildUIJSON_ErrorCreatePayment {
    my $self = shift;
    my $CreatePaymentService = $self->_createCreatePaymentService_Error();
    my $PatchPaymentService = $self->_createPatchPaymentService();
    return $self->_createUIJSON($CreatePaymentService, $PatchPaymentService);
}

sub _buildUIJSON_ErrorPatchPayment {
    my $self = shift;
    my $CreatePaymentService = $self->_createCreatePaymentService();
    my $PatchPaymentService = $self->_createPatchPaymentService_Error();
    return $self->_createUIJSON($CreatePaymentService, $PatchPaymentService);
}

sub _createUIJSON {
    my $self = shift;
    my ($CreatePaymentService, $PatchPaymentService) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::UI::JSON', [] );
    $Mockify->addMockWithReturnValue('_createCreatePaymentService', $CreatePaymentService);
    $Mockify->addMockWithReturnValue('_createPatchPaymentService', $PatchPaymentService);
    $Mockify->addMockWithReturnValueAndParameterCheck('_BuildShopUrl', 'some/url', [
        {'object' => 'DE_EPAGES::Shop::API::Object::Shop'},
        'hashref',
        {'hashref' => {'Type' => 'sf/mobile', 'UseSSL' => 1, 'AddToken' => 1}},
        {'object' => 'DE_EPAGES::WebInterface::Servlet::BaseServlet'},
    ]);
    $Mockify->addMockWithReturnValueAndParameterCheck('_GenerateDereferrerToken', 'token', [{'string'=>'some/url'}]);
    $Mockify->addMockWithReturnValueAndParameterCheck('writeContentOutput', undef ,['hashref']);
    my $Context = $self->_createContextLogger();
    $Mockify->addMockWithReturnValue('_createLogContext', $Context);

    my $UI_JSON = $Mockify->getMockObject();
    $UI_JSON->{'__ToCheck'}->{'_ContextLogger'} = $Context;
    return $UI_JSON;
}

sub _createContextLogger {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::ContextLogger', [] );
    $Mockify->addMock('logContext',
        sub {
            my $InternalSelf = shift;
            $InternalSelf->{'__ToCheck'}->{'_Params_logContext'} = \@_;
            my $FunctionPointer = $InternalSelf->{'__ToCheck'}->{'_Params_logContext'}[2];
            return $FunctionPointer->();
        }
    );
    return $Mockify->getMockObject();
}

sub _buildUIJSON_With_PaymentWallDataBuilder {
    my $self = shift;
    my ($PaymentWallDataBuilder) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
               'DE_EPAGES::PayPalPlus::UI::JSON',
               []
           );
    $Mockify->addMockWithReturnValue('_createPaymentWallDataBuilder', $PaymentWallDataBuilder);
    $Mockify->addMockWithReturnValueAndParameterCheck('writeContentOutput', undef, [{'hashref' => {'some'=>'hash'}}]);
    return $Mockify->getMockObject();
}

sub _createPaymentWallDataBuilder {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
           'DE_EPAGES::PayPalPlus::API::PaymentWallDataBuilder',
           []
        );
    $Mockify->addMockWithReturnValueAndParameterCheck('extractData', 1, [{'object'=>'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus'}]);
    $Mockify->addMockWithReturnValue('getData', {'some'=>'hash'});
    return $Mockify->getMockObject();
}

sub _createPaymentWallDataBuilder_Error {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
           'DE_EPAGES::PayPalPlus::API::PaymentWallDataBuilder',
           []
        );
    $Mockify->addMockWithReturnValueAndParameterCheck('extractData', 0, [{'object'=>'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus'}]);
    $Mockify->addMockWithReturnValue('getError', {'some'=>'hash'});
    return $Mockify->getMockObject();
}

sub _createCreatePaymentService {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
               'DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Service',
               []
           );
    $Mockify->addMockWithReturnValueAndParameterCheck('doRequest', 1, [
        {'object' => 'DE_EPAGES::Order::API::Object::Basket'},
        {'string' => 'some/url&DereferrerToken=token'},
        {'string' => 'some/url&DereferrerToken=token'},
    ]);
    my $CreatePaymentService = $Mockify->getMockObject();
    $self->{'ToCheck'}->{'CreatePaymentService'} = $CreatePaymentService;
    return $CreatePaymentService;
}

sub _createCreatePaymentService_Error {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
               'DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Service',
               []
           );
    $Mockify->addMockWithReturnValueAndParameterCheck('doRequest', 0, [
        {'object' => 'DE_EPAGES::Order::API::Object::Basket'},
        {'string' => 'some/url&DereferrerToken=token'},
        {'string' => 'some/url&DereferrerToken=token'},
    ]);
    my $CreatePaymentService = $Mockify->getMockObject();
    $self->{'ToCheck'}->{'CreatePaymentService'} = $CreatePaymentService;
    return $CreatePaymentService;
}

sub _createPatchPaymentService {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
               'DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::Service',
               []
           );
    $Mockify->addMockWithReturnValueAndParameterCheck('doRequest', 1, [
        {'object' => 'DE_EPAGES::Order::API::Object::Basket'},
    ]);
    my $PatchPaymentService = $Mockify->getMockObject();
    $self->{'ToCheck'}->{'PatchPaymentService'} = $PatchPaymentService;
    return $PatchPaymentService;
}

sub _createPatchPaymentService_Error {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
               'DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::Service',
               []
           );
    $Mockify->addMockWithReturnValueAndParameterCheck('doRequest', 0, [
        {'object' => 'DE_EPAGES::Order::API::Object::Basket'},
    ]);
    my $PatchPaymentService = $Mockify->getMockObject();
    $self->{'ToCheck'}->{'PatchPaymentService'} = $PatchPaymentService;
    return $PatchPaymentService;
}
__PACKAGE__->runScript();