#========================================================================================
# package      DE_EPAGES::Product::t::unit::ProductCollectionResource
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Product::API::Resource::ProductCollection
#========================================================================================
package DE_EPAGES::Product::t::unit::ProductCollectionResource;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Product::API::Resource::ProductCollection;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::Mockery;
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_get_errorOnTooManyGUIDs();

    $self->test_post_Product();

    done_testing();
}



sub test_get_errorOnTooManyGUIDs {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ProductCollectionResource = DE_EPAGES::Product::API::Resource::ProductCollection->new();
    my $aProductGUIDs = $self->_createGUIDs(101); ## no critic (ProhibitMagicNumbers)
    my $Mockery = $self->_createMocks();
    my $Shop = $Mockery->get('Shop');

    throws_ok{
        $ProductCollectionResource->get($Shop, 'de_DE', 'EUR', $aProductGUIDs);
    } qr/BadRequest: Reason=TooManyProductIds; ShopAlias=TestShop/,
    "$SubTestName - test if error is thrown when asking for more than 100 GUIDs";
}



sub test_post_Product {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $Resource = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');

    my $hRequestBody = {
        'test' => 123
    };

    my $hProductData = $Resource->post($Shop, 'de_DE', 'EUR', $hRequestBody);

    my $aCreateProductArgs = GetParametersFromMockifyCall($Mockery->get('ProductService'), 'createProduct');
    is_deeply($aCreateProductArgs->[3], $hRequestBody, "$SubTestName - test if createProduct was called with request body as argument.");
    is_deeply($hProductData, $self->_createProductData(), "$SubTestName - test correct product data was returned");
}



sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['Context', 'ProductService'], sub {
        my ($Context, $ProductService) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::Resource::ProductCollection', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createShopRequestContext', $Context, ['object']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createProductService', $ProductService, []);
        $Mockify->addMock('_RunBatchProcess', sub {
            my $MockedSelf = shift;
            my ($BatchTaskName, $cSub) = @_;
            $cSub->();
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('Shop', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'Alias' => 'TestShop',
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('Context', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::ShopRequestContext', [$Shop]);
        $Mockify->addMockWithReturnValueAndParameterCheck('validCurrency', 'EUR', [{'string' => 'EUR'}]);
        $Mockify->addMockWithReturnValueAndParameterCheck('validLocale', 'de_DE', [{'string' => 'de_DE'}]);
        return $Mockify->getMockObject();
    });
    $Mockery->register('ProductService', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::ProductService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('createProduct', $self->_createProductData(), ['object', 'string', 'string', 'hashref']);
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}



sub _createGUIDs {
    my $self = shift;
    my ($GuidCount) = @_;

    my $aGUIDs = [];
    for(my $i=0; $i<$GuidCount; $i++) {
        push(@$aGUIDs, $i);
    }

    return $aGUIDs;
}



sub _createProductData {
    return {
        'productid' => 'testproduct'
    };
}

__PACKAGE__->runScript();