#========================================================================================
# package      DE_EPAGES::WebService::t::unit::CodecJSON
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::WebService::API::Codec::JSON
#========================================================================================
package DE_EPAGES::WebService::t::unit::CodecJSON;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::WebService::API::Codec::JSON;
use HTTP::Message;
use HTTP::Headers;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_contentType();
    $self->test_encode();
    $self->test_decode();

    done_testing();
}



sub test_contentType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Codec = DE_EPAGES::WebService::API::Codec::JSON->new();

    is($Codec->contentType(), 'application/json', "$SubTestName - test if the correct content-type is returned.");
}



sub test_encode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Codec = DE_EPAGES::WebService::API::Codec::JSON->new();

    my $hData = {
        'key' => 'value',
        'msg' => ['hello', 'world'],
        'numbers' => [1,2,3],
        'visible' => JSON::XS::true,
    };
    my $Message = HTTP::Message->new();
    $Codec->encode($Message, $hData);

    is($Message->content(), '{"msg":["hello","world"],"visible":true,"numbers":[1,2,3],"key":"value"}', "$SubTestName - test if hashref was encoded correctly.");
    is($Message->content_type, 'application/json', "$SubTestName - test if the correct content-type is returned.");
}



sub test_decode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Codec = DE_EPAGES::WebService::API::Codec::JSON->new();

    my $String = '{"msg":["hello","world"],"numbers":[1,2,3],"key":"value", "visible": true}';
    my $Headers = HTTP::Headers->new();
    $Headers->header('Content-Length' => 1);
    my $Message = HTTP::Message->new($Headers, $String );
    my $hHash = $Codec->decode($Message);

    my $hExpectedData = {
        'key' => 'value',
        'msg' => ['hello', 'world'],
        'numbers' => [1,2,3],
        'visible' => 'true',
    };
    is_deeply($hHash, $hExpectedData, "$SubTestName - test if string was decoded correctly.");
}

__PACKAGE__->runScript();